unit cSPort;

interface

uses Windows, SysUtils, Dialogs, Math;

type
  TRTSUsed = (rtsNone, rtsUse, rtsInvert, rtsToggle);

  TSerialPort = class
  private
    CommTimeoutsRec: COMMTIMEOUTS;
    cp: DCB;
    hComm: DWORD;
    o: OVERLAPPED;
    RTSUsed: TRTSUsed;
  public
    TimeOut1, TimeOut2: Cardinal;

    constructor Create(const CisloCOMu: string; const BaudRate,
        Parity, ByteSize, StopBits: integer; const RTSUsed: TRTSUsed);
    destructor Destroy; override;
    function Transmit(var Buffer: array of byte; var Delka: Cardinal): Boolean;
    function Receive(var Buffer: array of byte; var Delka: Cardinal): Boolean;
  end;

implementation


constructor TSerialPort.Create(const CisloCOMu: string; const BaudRate,
    Parity, ByteSize, StopBits: integer; const RTSUsed: TRTSUsed);

  function RoundUp(const Num: Extended): Integer;
  begin
    Result := Trunc(Num);
    if Result < Num then Inc(Result);
  end;

begin
  TimeOut1 := 1;     // konstanty pro ovladani RTS
  TimeOut2 := 2;
  Self.RTSUsed := RTSUsed;

  hComm := CreateFile(PChar(CisloComu),
   GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING,
   FILE_ATTRIBUTE_NORMAL, 0);

  if hComm = INVALID_HANDLE_VALUE then
    MessageDlg('Chyba (#1) pi inicializaci sriovho portu "' + CisloComu
        + '"! Uzavete vechny aplikace, kter by ho mohly pouvat a restartujte program.',
        mtError, [mbOk], 0);

  CommTimeoutsRec.ReadIntervalTimeout := MAXDWORD;
      { bezpenost -> 2* + zaokrouhlit nahoru }
      { 8b + Start + Stop + Parity bit = 11b };
  CommTimeoutsRec.ReadTotalTimeoutMultiplier := 0;
  CommTimeoutsRec.ReadTotalTimeoutConstant := 0;
  CommTimeoutsRec.WriteTotalTimeoutMultiplier := 0;
  CommTimeoutsRec.WriteTotalTimeoutConstant := 0;
  if not SetCommTimeouts(hComm, CommTimeoutsRec) then
    MessageDlg('Chyba (#2) pi inicializaci sriovho portu "' + CisloComu
        + '"! Uzavete vechny aplikace, kter by ho mohly pouvat a restartujte program.',
        mtError, [mbOk], 0);

  cp.DCBlength := sizeof(cp);
  cp.BaudRate := BaudRate;

  cp.Flags := 1;  // binary mode
  cp.Flags := cp.Flags or (1 * Trunc(Power(2,1)));  // parity checking
  case RTSUsed of
    rtsUse:
      cp.Flags := cp.Flags or (RTS_CONTROL_DISABLE * Trunc(Power(2,12)));
    rtsToggle:
      cp.Flags := cp.Flags or (RTS_CONTROL_TOGGLE * Trunc(Power(2,12)));
    else
      cp.Flags := cp.Flags or (RTS_CONTROL_ENABLE * Trunc(Power(2,12)));
  end;

  cp.wReserved := 0;
  cp.XonLim := 0;
  cp.XoffLim := 0;
  cp.ByteSize := ByteSize;
  cp.Parity := Parity;
  cp.StopBits := StopBits;
  cp.XonChar := ' ';
  cp.XoffChar := ' ';
  cp.ErrorChar := ' ';
  cp.EofChar := ' ';
  cp.EvtChar := ' ';
  cp.wReserved1 := 0;
  if not SetCommState(hComm,cp) then
    MessageDlg('Chyba (#3) pi inicializaci sriovho portu "' + CisloComu
        + '"! Uzavete vechny aplikace, kter by ho mohly pouvat a restartujte program.',
        mtError, [mbOk], 0);

  SetCommMask(hComm, EV_TXEMPTY);
  o.hEvent := CreateEvent(nil, False, False, nil);
end;

destructor TSerialPort.Destroy;
begin
  CloseHandle(o.hEvent);
  CloseHandle(hComm);
  inherited;
end;

function TSerialPort.Transmit(var Buffer: array of byte; var Delka: Cardinal): Boolean;
var
  Written: Cardinal;
  lpfdwEvtMask: DWORD;
begin
  Result := True;

  if (RTSUsed = rtsUse) or (RTSUsed = rtsInvert) then
  begin
    if RTSUsed = rtsUse then
      EscapeCommFunction(hComm, SETRTS)
    else if RTSUsed = rtsInvert then
      EscapeCommFunction(hComm, CLRRTS);

    if TimeOut1 <> 0 then Sleep(TimeOut1);
  end;

  if not WriteFile(hComm, Buffer, Delka, Written, nil) then
    Result := False;
  if Written = 0 then Result := False;

  if (RTSUsed = rtsUse) or (RTSUsed = rtsInvert) then
  begin
    if Result then repeat
      WaitCommEvent(hComm, lpfdwEvtMask, @o);
    until (lpfdwEvtMask and EV_TXEMPTY) = EV_TXEMPTY;

    if TimeOut2 <> 0 then Sleep(TimeOut2);

    if RTSUsed = rtsUse then
      EscapeCommFunction(hComm, CLRRTS)
    else if RTSUsed = rtsInvert then
      EscapeCommFunction(hComm, SETRTS);
  end;
end;

function TSerialPort.Receive(var Buffer: array of byte; var Delka: Cardinal): Boolean;
begin
  Result := True;
  if not ReadFile(hComm, Buffer, sizeof(Buffer), Delka, nil) then
    Result := False;

  if Delka=0 then
  begin
    Result := False;
    Exit;
  end;
end;

end.
